@@ -9,6 +9,7 @@
 
 #include <QObject>
 #include <QQuickItem>
+#include <qqmlregistration.h>
 
 #include <KConfigWatcher>
 #include <KIO/ApplicationLauncherJob>
@@ -22,11 +23,12 @@
 class ShellUtil : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
     Q_PROPERTY(bool isSystem24HourFormat READ isSystem24HourFormat NOTIFY isSystem24HourFormatChanged)
 
 public:
     ShellUtil(QObject *parent = nullptr);
-    static ShellUtil *instance();
 
     /**
      * Change the stacking order to have the first item behind the second item.
@@ -52,7 +54,7 @@ public:
     Q_INVOKABLE void executeCommand(const QString &command);
 
     /**
-     * Launch an application by name. Sets the internal "launched app" state.
+     * Launch an application by name.
      *
      * @param storageId The storage id of the application to launch.
      */
