@@ -7,13 +7,9 @@
 #include <Plasma/Containment>
 #include <QSortFilterProxyModel>
 
-#include <applicationlistmodel.h>
-#include <desktopmodel.h>
-
 class HomeScreen : public Plasma::Containment
 {
     Q_OBJECT
-    Q_PROPERTY(bool showingDesktop READ showingDesktop WRITE setShowingDesktop NOTIFY showingDesktopChanged)
 
 public:
     HomeScreen(QObject *parent, const KPluginMetaData &data, const QVariantList &args);
@@ -21,9 +17,10 @@ public:
 
     void configChanged() override;
 
-    bool showingDesktop() const;
-    void setShowingDesktop(bool showingDesktop);
-
 Q_SIGNALS:
     void showingDesktopChanged(bool showingDesktop);
+
+private Q_SLOTS:
+    void onAppletAdded(Plasma::Applet *applet, const QRectF &geometryHint);
+    void onAppletAboutToBeRemoved(Plasma::Applet *applet);
 };
